@@ -13,6 +13,7 @@
 #include <QObject>
 
 #include "dmabufhandler.h"
+#include "pipewireproduce_p.h"
 
 extern "C" {
 #include "libavcodec/avcodec.h"
@@ -95,6 +96,10 @@ public:
      */
     void setQuality(std::optional<quint8> quality);
 
+    static bool supportsHardwareEncoding();
+
+    void setEncodingPreference(PipeWireBaseEncodedStream::EncodingPreference preference);
+
 protected:
     virtual int percentageToAbsoluteQuality(const std::optional<quint8> &quality) = 0;
 
@@ -108,6 +113,8 @@ protected:
     AVFilterContext *m_outputFilter = nullptr;
 
     std::optional<quint8> m_quality;
+    PipeWireBaseEncodedStream::EncodingPreference m_encodingPreference;
+    virtual void applyEncodingPreference(AVDictionary *options);
 };
 
 /**
@@ -167,7 +174,4 @@ protected:
 
     AVBufferRef *m_drmContext = nullptr;
     AVBufferRef *m_drmFramesContext = nullptr;
-
-private:
-    bool m_supportsHardwareModifiers;
 };
