@@ -184,24 +184,39 @@ static QHash<spa_video_format, QList<uint64_t>> queryDmaBufModifiers(EGLDisplay
         }
 
         QList<uint64_t> queriedModifiers(count);
+        QList<EGLBoolean> externalOnly(count);
         if (count > 0) {
-            if (!eglQueryDmaBufModifiersEXT(display, drm_format, count, queriedModifiers.data(), nullptr, &count)) {
+            if (!eglQueryDmaBufModifiersEXT(display, drm_format, count, queriedModifiers.data(), externalOnly.data(), &count)) {
                 qCWarning(PIPEWIRE_LOGGING) << "Failed to query DMA-BUF modifiers.";
             }
         }
 
-        QList<uint64_t> usableModifiers(count + 1);
+        QList<uint64_t> usableModifiers;
+        usableModifiers.reserve(count + 1);
         if (usageHint == PipeWireSourceStream::UsageHint::EncodeHardware) {
             auto vaapi = VaapiUtils::instance();
-            std::copy_if(queriedModifiers.begin(), queriedModifiers.end(), std::back_inserter(usableModifiers), [vaapi, drm_format](uint64_t modifier) {
-                return vaapi->supportsModifier(drm_format, modifier);
-            });
+            for (int i = 0; i < queriedModifiers.size(); ++i) {
+                if (externalOnly[i]) {
+                    continue;
+                }
+                const uint64_t modifier = queriedModifiers[i];
+                if (vaapi->supportsModifier(drm_format, modifier)) {
+                    usableModifiers.append(modifier);
+                }
+            }
         } else {
-            usableModifiers = queriedModifiers;
+            for (int i = 0; i < queriedModifiers.size(); ++i) {
+                if (!externalOnly[i]) {
+                    usableModifiers.append(queriedModifiers[i]);
+                }
+            }
+        }
+
+        if (!usableModifiers.isEmpty()) {
+            // Support modifier-less buffers
+            usableModifiers.push_back(DRM_FORMAT_MOD_INVALID);
         }
 
-        // Support modifier-less buffers
-        usableModifiers.push_back(DRM_FORMAT_MOD_INVALID);
         ret[format] = usableModifiers;
     }
     return ret;
@@ -281,11 +296,7 @@ buildFormat(spa_pod_builder *builder, spa_video_format format, const QList<uint6
         spa_pod_builder_add(builder, SPA_FORMAT_VIDEO_framerate, SPA_POD_CHOICE_RANGE_Fraction(&defFramerate, &defFramerate, &maxFramerate), 0);
     }
 
-    if (modifiers.size() == 1 && modifiers[0] == DRM_FORMAT_MOD_INVALID) {
-        // we only support implicit modifiers, use shortpath to skip fixation phase
-        spa_pod_builder_prop(builder, SPA_FORMAT_VIDEO_modifier, SPA_POD_PROP_FLAG_MANDATORY);
-        spa_pod_builder_long(builder, modifiers[0]);
-    } else if (!modifiers.isEmpty()) {
+    if (!modifiers.isEmpty()) {
         // SPA_POD_PROP_FLAG_DONT_FIXATE can be used with PipeWire >= 0.3.33
         if (withDontFixate) {
             spa_pod_builder_prop(builder, SPA_FORMAT_VIDEO_modifier, SPA_POD_PROP_FLAG_MANDATORY | SPA_POD_PROP_FLAG_DONT_FIXATE);
